<?php

/**
 * @file
 * Page for the standings.
 */

/**
 * Tournament_series standings page.
 *
 * @param int $arg1
 *   "@param" is series.id read  in from path "standings/%"
 *
 * @return array
 *   Theme table sorted according to the ranking ASC
 *   'Rank', 'Player', 'Nation', 'RO1'..'RO-last'
 */
function tournaments_series_standings($arg1) {
  if (is_numeric($arg1)) {
    // Must load the rules for the series so it could put all ranking options.
    $query = db_select('tournaments_series_series', 'ts');
    $query->join('tournaments_series_rules', 'r', 'ts.rid = r.id');
    $query->fields('r');
    $query->condition('ts.id', $arg1);
    $result = $query->execute();
    if ($result->rowCount()) {
      $record = $result->fetchAssoc();
      $rules_titles = array();
      foreach (unserialize($record['options']) as $key => $rule) {
        if (check_plain($rule['rule']) == 'mutual') {
          $rules_titles[$key] = t('Mutual');
        }
        else {
          $ii = check_plain($rule['from']) + check_plain($rule['count']) - 1;
          if (check_plain($rule['count']) == '0') {
            $_rules_titles = t('Best') . ' ' . check_plain($rule['from']) . '. - all';
            $rules_titles[$key] = $_rules_titles;
          }
          elseif (check_plain($rule['count']) == '1') {
            $_rules_titles = t('Best') . ' ' . check_plain($rule['from']) . '.';
            $rules_titles[$key] = $_rules_titles;
          }
          else {
            $_rules_titles = t('Best') . ' ' . check_plain($rule['from']) . '. - ' . $ii . '.';
            $rules_titles[$key] = $_rules_titles;
          }
          if (check_plain($rule['from']) == '1' && check_plain($rule['count']) > '1') {
            $_rules_titles = t('Best') . ' ' . $rule['count'];
            $rules_titles[$key] = $_rules_titles;
          }
          if (check_plain($rule['count']) == '0' && check_plain($rule['from']) == '1') {
            $rules_titles[$key] = t('All');
          }
        }
      }

      $header = array(t('Rank'), t('Player'), t('Nation'));
      $query = db_select('tournaments_series_standings', 'st');
      $query->join('tournaments_series_players', 'p', 'p.id = st.pid');
      $query->join('tournaments_series_series', 's', 's.id = st.sid');
      $query->fields('st')
          ->fields('p', array('player', 'nation'))
          ->fields('s', array('name'))
          ->extend('PagerDefault')
          ->orderBy('rank', 'ASC')
          ->limit(50);
      $query->condition('sid', $arg1);
      $result = $query->execute();
      if ($result->rowCount()) {
        $standing = array();
        foreach ($result as $key => $row) {
          $standing[$key]['rank'] = $row->rank;
          $standing[$key]['player'] = check_plain($row->player);
          $standing[$key]['nation']['data'] = tournaments_series_helper_get_land_name_from_ioc_code(check_plain($row->nation));
          $standing[$key]['nation']['class'] = 'nation-' . strtolower($row->nation);
          $caption = $row->name;
          for ($i = 1; $i <= count($rules_titles); $i++) {
            $ro = 'ro' . $i;
            $standing[$key][$ro] = $row->$ro;
            $header[$i + 2] = t($rules_titles[$i]);
          }
        }
        $output = theme('table', array(
            'header' => $header,
            'rows' => $standing,
            'caption' => $caption,
          )
        ) . theme('pager');
      }
      else {
        $output = t("Sorry, but I can't find this series");
      }
    }
    else {
      $output = t("Sorry, but I can't find this series");
    }
  }
  else {
    $output = t("Sorry, but I can't find this series");
  }
  return $output;
}
