<?php

/**
 * @file
 * All the main logic of this module is in this file.
 */

/**
 * Inserts or update fields in  table.
 *
 * @param array $rule
 *   array of rules.
 *
 * @param string $table
 *   string width table name
 *
 * @return int
 *   inserted ID
 */
function tournaments_series_helper_insert($rule, $table) {
  if (isset($rule['id'])) {
    return db_update($table)
            ->fields($rule)
            ->condition('id', $rule['id'])
            ->execute();
  }
  else {
    unset($rule['id']);
    return db_insert($table)
            ->fields($rule)
            ->execute();
  }
}

/**
 * A function that checks if a player is on the tournement.
 *
 * @param int $id
 *   player id
 *
 * @return boolean
 *   TRUE if the player id is in a played tournament, and FALSE if it isn't
 */
function tournaments_series_helper_played($id) {
  $result = db_query(
      "SELECT * FROM {tournaments_series_tournaments} WHERE played = '1'"
  );
  foreach ($result as $row) {
    $players = unserialize($row->ranking);
    foreach ($players as $player) {
      if ($player == $id) {
        return TRUE;
        break;
      }
    }
  }
  return FALSE;
}

/**
 * A function that deletes tha table.
 *
 * @param int $id
 *   data base table id
 *
 * @param string $table
 *   data base table name
 *
 * @return bool
 *   TRUE if deleted, and FALSE if it isn't
 */
function tournaments_series_helper_delete_tournaments_series($id, $table) {
  $test_return = db_delete($table)
      ->condition('id', $id)
      ->execute();
  // Here it must delete
  // if there is a connection between the tournament & series.
  if ($test_return) {
    if ($table == 'tournaments_series_tournaments') {
      db_delete('tournaments_series_tournament_serie')
          ->condition('tid', $id)
          ->execute();
    }
    if ($table == 'tournaments_series_series') {
      db_delete('tournaments_series_tournament_serie')
          ->condition('sid', $id)
          ->execute();

      db_delete('tournaments_series_standings')
          ->condition('sid', $id)
          ->execute();
    }

    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * Load names form base.
 *
 * @param string $db_table
 *   data base table name
 *
 * @return array
 *   row "name"
 */
function tournaments_series_helper_load_names($db_table) {
  $r = array();
  $result = db_query(
      "SELECT id, name FROM {$db_table}"
  );
  foreach ($result as $row) {
    $r[$row->id] = check_plain($row->name);
  }
  return $r;
}

/**
 * Load series id's form "tournaments_series_tournament_serie" table in base.
 *
 * @param int $tournament_id
 *   tournament id
 *
 * @return array
 *   row "sid"
 */
function tournaments_series_helper_load_id($tournament_id) {
  $r = array();
  $result = db_query(
      "SELECT * FROM {tournaments_series_tournament_serie} WHERE tid = :tournament_id", array(':tournament_id' => $tournament_id)
  );
  foreach ($result as $row) {
    $r[] = $row->sid;
  }
  return $r;
}

/**
 * Load players names form base.
 *
 * @return array
 *   row "player"
 */
function tournaments_series_helper_load_players_names() {
  $r = array();
  $result = db_query(
      "SELECT * FROM {tournaments_series_players}"
  );
  foreach ($result as $row) {
    $r[] = check_plain($row->player);
  }
  return $r;
}

/**
 * Return array from textarea.
 *
 * @param array $textarea
 *   Textarea post or get data.
 *
 * @return array
 *   Array of lines
 */
function tournaments_series_helper_explode_textarea($textarea) {
  $a = $textarea;
  $a = str_replace(array("\r\n", "\t"), array("[NEW*LINE]", "[tAbul*Ator]"), $a);
  $line_count = 1;
  $data = array();
  foreach (explode("[NEW*LINE]", $a) as $lines) {
    $data[$line_count] = $lines;
    $line_count++;
  }
  return $data;
}

/**
 * Text area from array.
 */
function tournaments_series_helper_implode_textarea($area) {
  $textarea = '';
  foreach ($area as $row) {
    if ($textarea != '') {
      $textarea .= "\r\n";
    }
    $textarea .= $row;
  }
  return $textarea;
}

/**
 * Return array ioc codes.
 *
 * @return array
 *   array of ioc codes
 */
function tournaments_series_helper_ioc_codes() {
  return array(
    'AFG' => t('Afghanistan'),
    'ALB' => t('Albania'),
    'ALG' => t('Algeria'),
    'ASA' => t('American Samoa*'),
    'AND' => t('Andorra'),
    'ANG' => t('Angola'),
    'ANT' => t('Antigua and Barbuda'),
    'ARG' => t('Argentina'),
    'ARM' => t('Armenia'),
    'ARU' => t('Aruba*'),
    'AUS' => t('Australia'),
    'AUT' => t('Austria'),
    'AZE' => t('Azerbaijan'),
    'BAH' => t('The Bahamas'),
    'BRN' => t('Bahrain'),
    'BAN' => t('Bangladesh'),
    'BAR' => t('Barbados'),
    'BLR' => t('Belarus'),
    'BEL' => t('Belgium'),
    'BIZ' => t('Belize'),
    'BER' => t('Bermuda*'),
    'BEN' => t('Benin'),
    'BHU' => t('Bhutan'),
    'BOL' => t('Bolivia'),
    'BIH' => t('Bosnia and Herzegovina'),
    'BOT' => t('Botswana'),
    'BRA' => t('Brazil'),
    'IVB' => t('British Virgin Islands*'),
    'BRU' => t('Brunei'),
    'BUL' => t('Bulgaria'),
    'BUR' => t('Burkina Faso'),
    'BDI' => t('Burundi'),
    'CAM' => t('Cambodia'),
    'CMR' => t('Cameroon'),
    'CAN' => t('Canada'),
    'CPV' => t('Cape Verde'),
    'CAY' => t('Cayman Islands*'),
    'CAF' => t('Central African Republic'),
    'CHA' => t('Chad'),
    'CHI' => t('Chile'),
    'CHN' => t('China'),
    'COL' => t('Colombia'),
    'COM' => t('Comoros'),
    'CGO' => t('Congo, Republic of the'),
    'COD' => t('Congo, Democratic Republic of the'),
    'COK' => t('Cook Islands*'),
    'CRC' => t('Costa Rica'),
    'CIV' => t("Cote d'Ivoire"),
    'CRO' => t('Croatia'),
    'CUB' => t('Cuba'),
    'CYP' => t('Cyprus'),
    'CZE' => t('Czech Republic'),
    'DEN' => t('Denmark'),
    'DJI' => t('Djibouti'),
    'DMA' => t('Dominica'),
    'DOM' => t('Dominican Republic'),
    'TLS' => t('East Timor (Timor Leste)'),
    'ECU' => t('Ecuador'),
    'EGY' => t('Egypt'),
    'ESA' => t('El Salvador'),
    'GEQ' => t('Equatorial Guinea'),
    'ERI' => t('Eritrea'),
    'EST' => t('Estonia'),
    'ETH' => t('Ethiopia'),
    'FIJ' => t('Fiji'),
    'FIN' => t('Finland'),
    'FRA' => t('France'),
    'GAB' => t('Gabon'),
    'GAM' => t('The Gambia'),
    'GEO' => t('Georgia'),
    'GER' => t('Germany'),
    'GHA' => t('Ghana'),
    'GRE' => t('Greece'),
    'GRN' => t('Grenada'),
    'GUM' => t('Guam*'),
    'GUA' => t('Guatemala'),
    'GUI' => t('Guinea'),
    'GBS' => t('Guinea Bissau'),
    'GUY' => t('Guyana'),
    'HAI' => t('Haiti'),
    'HON' => t('Honduras'),
    'HKG' => t('Hong Kong*'),
    'HUN' => t('Hungary'),
    'ISL' => t('Iceland'),
    'IND' => t('India'),
    'INA' => t('Indonesia'),
    'IRI' => t('Iran'),
    'IRQ' => t('Iraq'),
    'IRL' => t('Ireland'),
    'ISR' => t('Israel'),
    'ITA' => t('Italy'),
    'JAM' => t('Jamaica'),
    'JPN' => t('Japan'),
    'JOR' => t('Jordan'),
    'KAZ' => t('Kazakhstan'),
    'KEN' => t('Kenya'),
    'PRK' => t('Korea, North (PDR of Korea)'),
    'KOR' => t('Korea, South'),
    'KUW' => t('Kuwait'),
    'KGZ' => t('Kyrgyzstan'),
    'LAO' => t('Laos'),
    'LAT' => t('Latvia'),
    'LIB' => t('Lebanon'),
    'LES' => t('Lesotho'),
    'LBR' => t('Liberia'),
    'LBA' => t('Libya'),
    'LIE' => t('Liechtenstein'),
    'LTU' => t('Lithuania'),
    'LUX' => t('Luxembourg'),
    'MKD' => t('Macedonia'),
    'MAD' => t('Madagascar'),
    'MAW' => t('Malawi'),
    'MAS' => t('Malaysia'),
    'MDV' => t('Maldives'),
    'MLI' => t('Mali'),
    'MLT' => t('Malta'),
    'MTN' => t('Mauritania'),
    'MRI' => t('Mauritius'),
    'MEX' => t('Mexico'),
    'FSM' => t('Federated States of Micronesia'),
    'MDA' => t('Moldova'),
    'MON' => t('Monaco'),
    'MGL' => t('Mongolia'),
    'MAR' => t('Morocco'),
    'MOZ' => t('Mozambique'),
    'MYA' => t('Myanmar (Burma)'),
    'NAM' => t('Namibia'),
    'NRU' => t('Nauru'),
    'NEP' => t('Nepal'),
    'NED' => t('Netherlands'),
    'AHO' => t('Netherlands Antilles*'),
    'NZL' => t('New Zealand'),
    'NCA' => t('Nicaragua'),
    'NIG' => t('Niger'),
    'NGR' => t('Nigeria'),
    'NOR' => t('Norway'),
    'OMA' => t('Oman'),
    'PAK' => t('Pakistan'),
    'PLW' => t('Palau'),
    'PLE' => t('Palestine*'),
    'PAN' => t('Panama'),
    'PNG' => t('Papua New Guinea'),
    'PAR' => t('Paraguay'),
    'PER' => t('Peru'),
    'PHI' => t('Philippines'),
    'POL' => t('Poland'),
    'POR' => t('Portugal'),
    'PUR' => t('Puerto Rico*'),
    'QAT' => t('Qatar'),
    'ROM' => t('Romania'),
    'RUS' => t('Russian Federation'),
    'RWA' => t('Rwanda'),
    'SKN' => t('Saint Kitts and Nevis'),
    'LCA' => t('Saint Lucia'),
    'VIN' => t('Saint Vincent and the Grenadines'),
    'SAM' => t('Samoa'),
    'SMR' => t('San Marino'),
    'STP' => t('Sao Tome and Principe'),
    'KSA' => t('Saudi Arabia'),
    'SEN' => t('Senegal'),
    'SCG' => t('Serbia and Montenegro'),
    'SEY' => t('Seychelles'),
    'SLE' => t('Sierra Leone'),
    'SIN' => t('Singapore'),
    'SVK' => t('Slovakia'),
    'SLO' => t('Slovenia'),
    'SOL' => t('Solomon Islands'),
    'SOM' => t('Somalia'),
    'RSA' => t('South Africa'),
    'ESP' => t('Spain'),
    'SRI' => t('Sri Lanka'),
    'SUD' => t('Sudan'),
    'SUR' => t('Suriname'),
    'SWZ' => t('Swaziland'),
    'SWE' => t('Sweden'),
    'SUI' => t('Switzerland'),
    'SYR' => t('Syria'),
    'TPE' => t('Taiwan (Chinese Taipei)'),
    'TJK' => t('Tajikistan'),
    'TAN' => t('Tanzania'),
    'THA' => t('Thailand'),
    'TOG' => t('Togo'),
    'TGA' => t('Tonga'),
    'TRI' => t('Trinidad and Tobago'),
    'TUN' => t('Tunisia'),
    'TUR' => t('Turkey'),
    'TKM' => t('Turkmenistan'),
    'UGA' => t('Uganda'),
    'UKR' => t('Ukraine'),
    'UAE' => t('United Arab Emirates'),
    'GBR' => t('United Kingdom (Great Britain)'),
    'USA' => t('United States'),
    'URU' => t('Uruguay'),
    'UZB' => t('Uzbekistan'),
    'VAN' => t('Vanuatu'),
    'VEN' => t('Venezuela'),
    'VIE' => t('Vietnam'),
    'ISV' => t('Virgin Islands*'),
    'YEM' => t('Yemen'),
    'ZAM' => t('Zambia'),
    'ZIM' => t('Zimbabwe'),
  );
}

/**
 * Get name from IOC code.
 *
 * @param string $code
 *   IOC code ( AUS,CRO ...)
 *
 * @return string
 *   full name of the country
 */
function tournaments_series_helper_get_land_name_from_ioc_code($code) {
  $list = tournaments_series_helper_ioc_codes();
  if (array_key_exists($code, $list)) {
    return $list[$code];
  }
  else {
    return t('No nation code');
  }
}

/**
 * From the table 'tournaments_series_standings' deletes the current condition.
 *
 * @param int $serie
 *   Series Id
 */
function tournaments_series_helper_delete_standings($serie) {
  db_delete('tournaments_series_standings')
      ->condition('sid', $serie)
      ->execute();
}

/**
 * From the table 'tournaments_series_standings' deletes the current condition.
 *
 * @param int $serie
 *   Series Id
 *
 * @return array
 *   rules in array
 */
function tournaments_series_helper_load_rules($serie) {
  $rules = array();
  $result = db_query(
      "SELECT * FROM {tournaments_series_series}
          LEFT JOIN {tournaments_series_rules}
          ON (tournaments_series_series.rid = tournaments_series_rules.id)
          WHERE  tournaments_series_series.id = :serie", array(':serie' => $serie)
  );
  foreach ($result as $row) {
    $rules['points'] = tournaments_series_check_plain_array(unserialize($row->points));
    $rules['options'] = unserialize($row->options);
  }
  return $rules;
}

/**
 * For each player set up points by tournaments.
 *
 * @param int $serie
 *   serie id
 *
 * @param array $rules
 *   rules
 */
function tournaments_series_helper_load_players_points($serie, $rules) {
  // Loads the base with the tournaments which are in $serie with the results.
  $result = db_query(
      "SELECT * FROM {tournaments_series_tournaments}
          RIGHT JOIN {tournaments_series_tournament_serie}
          ON (tournaments_series_tournaments.id = tournaments_series_tournament_serie.tid)
          WHERE  tournaments_series_tournament_serie.sid = :serie
         ", array(':serie' => $serie)
  );
  // For each player must set up points by tournaments in $players_positions.
  $players_positions = array();
  foreach ($result as $row) {
    if ($row->played) {
      foreach (unserialize($row->ranking) as $position => $pid) {
        if (isset($rules['points'][$position + 1])) {
          $players_positions[$pid]['tournaments'][$row->id] = $rules['points'][$position + 1];
          $players_positions[$pid]['sorted points'][] = $rules['points'][$position + 1];
        }
        else {
          $players_positions[$pid]['tournaments'][$row->id] = 0;
          $players_positions[$pid]['sorted points'][] = 0;
        }
      }
    }
  }
  return $players_positions;
}

/**
 * Save standings in "tournaments_series_standings" table.
 *
 * @param array $players_positions
 *   All data in one array
 *
 * @param int $serie
 *   serie id
 */
function tournaments_series_helper_save_standing($players_positions, $serie) {
  $ranking = array();
  foreach ($players_positions as $player => $rank) {
    $ranking['pid'] = $player;
    $ranking['sid'] = $serie;
    $ranking['rank'] = $rank['rank'];
    foreach ($rank['ranking options'] as $rok => $ro) {
      $ranking['ro' . $rok] = $ro;
    }
    // At the end it saves all in the table.
    db_insert('tournaments_series_standings')
        ->fields($ranking)
        ->execute();
  }
}

/**
 * Check_plain_array.
 */
function tournaments_series_check_plain_array($array) {
  $results = array();
  if (!empty($array)) {
    foreach ($array as $key => $a) {
      $results[check_plain($key)] = check_plain($a);
    }
  }
  return $results;
}
/**
 * Main function for calculating standings.
 *
 * For all series it calculates standings
 * and saves them in the table tournaments_series_standings.
 *
 * @param array $series
 *   array of series
 */
function tournaments_series_helper_standings($series) {
  foreach ($series as $s) {
    // From the table 'tournaments_series_standings'
    // deletes the current condition.
    tournaments_series_helper_delete_standings($s);
    // Loads the rules for the season and puts them in array $rules.
    $rules = tournaments_series_helper_load_rules($s);
    // For each player must set up points by tournaments in $players_positions.
    $players_positions = array();
    $players_positions = tournaments_series_helper_load_players_points($s, $rules);
    // Before each rule must add a rank.
    // For each player at the beginning they all have rank 1.
    foreach ($players_positions as $player => $player_position) {
      $players_positions[$player]['rank'] = 1;
      // Now sorts all results for each player.
      rsort($players_positions[$player]['sorted points']);
    }
    foreach ($rules['options'] as $rules_key => $option) {
      foreach ($players_positions as $player_key => $player) {
        $players_positions[$player_key]['ranking options'][$rules_key] = 0;

        // *********************************************************
        // For each new option a link
        // with the adjoining function can be controlled here.
        // The option is needed to add
        // in a function  "tournaments_series_add_rules_form".:
        // example:
        // if ($option['rule'] == 'your_option') {
        // tournaments_series_helper_standings_your_option
        // (
        // $players_positions,
        // $player_key,
        // $player,
        // $option,
        // $rules_key
        // );
        // }
        if ($option['rule'] == 'sum') {
          tournaments_series_helper_standings_sum($players_positions, $player_key, $player, $option, $rules_key);
        }
        if ($option['rule'] == 'mutual') {
          tournaments_series_helper_standings_mutual($players_positions, $player_key, $player, $rules_key);
        }
        // *********************************************************
      }
      // At the end of each rule must set up ranking for each player.
      tournaments_series_helper_set_up_ranking($players_positions, $rules_key);
    }
    // At the end it saves all in the table.
    tournaments_series_helper_save_standing($players_positions, $s);
  }
  return;
}

/**
 * For each new option a link with the adjoining function can be added here.
 */
function tournaments_series_ranking_options_array() {
  return array(
    'sum' => 'Sums best results',
    'mutual' => 'Sums all mutual results',
  );
}

/**
 * At the end of each rule ("$rules_key")
 * must set up ranking ("$players_positions[$p_key]['rank']")
 * for each player.
 */
function tournaments_series_helper_set_up_ranking(&$players_positions, $rules_key) {
  foreach ($players_positions as $player_key_temp_a => $player_temp_a) {
    $better_positions[$player_key_temp_a] = 0;
    foreach ($players_positions as $player_key_temp_b => $player_temp_b) {
      if (($player_key_temp_a != $player_key_temp_b) &&
          ($player_temp_a['rank'] == $player_temp_b['rank']) &&
          ($player_temp_a['ranking options'][$rules_key] < $player_temp_b['ranking options'][$rules_key])) {
        $better_positions[$player_key_temp_a]++;
      }
    }
  }
  foreach ($players_positions as $p_key => $player) {
    $players_positions[$p_key]['rank'] += $better_positions[$p_key];
  }
  return;
}

/**
 * Different ranking functions are added here.
 *
 * Any variable available in
 *  "tournaments_series_helper_standings" function
 *  can be used as a parameter.
 *    It is important to have both
 *      &$players_positions
 *        variable that can be changed so that
 *        a "tournaments_series_helper_set_up_ranking" function
 *        can sort it correctly
 *    and
 *      $rules_key
 *        variable that is used as a key of array field
 *        in which the function adds its calculated numbers,
 *        e.g. $players_positions[$player_key]['ranking options'][$rules_key].
 *
 * The first function is "tournaments_series_helper_standings_sum",
 * it sums the best results.
 */
function tournaments_series_helper_standings_sum(&$players_positions, $player_key, $player, $option, $rules_key) {
  // Checks if a zero was written
  // for all results or if it just counts up to a specific number.
  if ($option['count']) {
    // Only the given number of  results.
    // From where counting starts.
    $ii = $option['from'] - 1;
    for ($i = $ii; $i < $option['count'] + $ii; $i++) {
      if (isset($players_positions[$player_key]['sorted points'][$i])) {
        $players_positions[$player_key]['ranking options'][$rules_key] += $players_positions[$player_key]['sorted points'][$i];
      }
    }
  }
  else {
    // All results.
    foreach ($player['sorted points'] as $i => $points) {
      // Controls from where the results are counted.
      // +2 is because it loads 1 from the first, and it is in array as 0.
      if ($option['from'] < $i + 2) {
        $players_positions[$player_key]['ranking options'][$rules_key] += $points;
      }
    }
  }
  return;
}

/**
 * Mutual option.
 *
 * For the players on the same positions it sums all the points,
 * but only from the tournaments on which all these players participated.
 */
function tournaments_series_helper_standings_mutual(&$players_positions, $player_key, $player, $rules_key) {
  $mutal_players_list = array();
  foreach ($players_positions as $test_player_id => $test_player) {
    if ($player_key != $test_player_id && $player['rank'] == $test_player['rank']) {
      $mutal_players_list[] = $test_player_id;
    }
  }
  $point_to_sum = array();
  foreach ($players_positions[$player_key]['tournaments'] as $tkey => $tournament_point) {
    $point_to_sum[$tkey] = !empty($mutal_players_list) ? $tournament_point : 0;
    foreach ($mutal_players_list as $mpid) {
      if (!isset($players_positions[$mpid]['tournaments'][$tkey])) {
        $point_to_sum[$tkey] = 0;
      }
    }
  }
  foreach ($point_to_sum as $point) {
    $players_positions[$player_key]['ranking options'][$rules_key] += $point;
  }
  return;
}
